/* compconn.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "grafo-fs.h"


#define ROW_LENGTH 256


/* Prototipi delle procedure secondarie e definizione di tipi */

typedef int *vint;

/* Legge dalla linea di comando il file che contiene il grafo */
void InterpretaLineaComando (int argc, char *argv[], char *filedati);

/* Carica dal file filedati il grafo G, assumendo che sia orientato o no secondo il valore di Orientato */
void CaricaGrafo (char *filedati, grafo *pG, boolean Orientato);

/* Stampa le nc componenti conservate nel vettore C di lunghezza n */
void StampaComponenti (int nc, vint C, int n);

/* Determina le nc componenti fortemente connesse del grafo G nel vettore C */
void ComponentiFortementeConnesse (grafo *pG, vint C, int *pnc);


/* Programma principale */
int main (int argc, char *argv[])
{
  char filedati[ROW_LENGTH];
  grafo G;
  vint C;
  int nc;


  /* Legge da linea di comando il file che contiene il grafo da visitare */
  InterpretaLineaComando(argc,argv,filedati);

  /* Carica il grafo */
  CaricaGrafo(filedati,&G,TRUE);

  /* Stampa a video il grafo */
  printf("Grafo: ");
  stampagrafo(&G);
  printf("\n");

  /* Crea una soluzione C vuota */
  nc = 0;
  C = (vint) calloc(G.n+1,sizeof(int));
  if (C == NULL)
  {
    fprintf(stderr,"Errore nell'allocazione del vettore C!\n");
    exit(EXIT_FAILURE);
  }

  /* Determina le nc componenti fortemente connesse del grafo G nel vettore C */
  ComponentiFortementeConnesse(&G,C,&nc);

  /* Stampa le componenti fortemente connesse del grafo G */
  StampaComponenti(nc,C,G.n);

  /* Dealloca le strutture dati dinamiche */
  distruggegrafo(&G);
  free(C);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

/* Legge dalla linea di comando il file che contiene il grafo */
void InterpretaLineaComando (int argc, char *argv[], char *filedati)
{
  if (argc != 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    exit(EXIT_FAILURE);
  }

  strcpy(filedati,argv[1]);
}


/* Carica dal file filedati il grafo G, assumendo che sia orientato o no secondo il valore di Orientato */
void CaricaGrafo (char *filedati, grafo *pG, boolean Orientato)
{
  FILE *fp;
  nodo o, d;
  int n;


  /* Apre il file */
  fp = fopen(filedati,"r");
  if (fp == NULL)
  {
    fprintf(stderr,"Errore nell'apertura del file %s!\n",filedati);
    exit(EXIT_FAILURE);
  }

  /* Conta i nodi del grafo */
  n = 0;
  while (fscanf(fp," (%d,%d)",&o,&d) == 2)
  {
    if (o > n) n = o;
    if (d > n) n = d;
  }

  /* Crea il grafo, assumendo che gli indici dei nodi vadano da 1 a n */
  creagrafo(n,pG);

  /* Aggiunge gli archi del grafo */
  rewind(fp);
  while (fscanf(fp," (%d,%d)",&o,&d) == 2)
  {
    insarco(o,d,pG);
    if (!Orientato) insarco(d,o,pG);
  }

  /* Chiude il file */
  fclose(fp);
}


/* Stampa le nc componenti conservate nel vettore di marcatura C di lunghezza n */
void StampaComponenti (int nc, vint C, int n)
{
  int v, c;

  printf("%d componenti\n",nc);
  printf("C = [ ");
  for (v = 1; v <= n; v++)
    printf("%d ",C[v]);
  printf("]\n");

  for (c = 1; c <= nc; c++)
  {
    printf("U[%d] = ",c);
    for (v = 1; v <= n; v++)
      if (C[v] == c) printf("%d ",v);
    printf("\n");
  }
}


/* Determina le nc componenti connesse del grafo G nel vettore C */
void ComponentiFortementeConnesse (grafo *pG, vint C, int *pnc)
{
}
